Package: socialmixr
Title: Social Mixing Matrices for Infectious Disease Modelling
Version: 0.6.0
Authors@R: c(
    person("Sebastian", "Funk", , "sebastian.funk@lshtm.ac.uk", role = c("aut", "cre")),
    person("Lander", "Willem", role = "aut"),
    person("Hugo", "Gruson", role = "aut"),
    person("Nicholas", "Tierney", , "nicholas.tierney@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-1460-8722")),
    person("Maria", "Bekker-Nielsen Dunbar", role = "ctb"),
    person("Carl A. B.", "Pearson", role = "ctb"),
    person("Sam", "Clifford", , "sj.clifford@gmail.com", role = "ctb"),
    person("Christopher", "Jarvis", role = "ctb"),
    person("Alexis", "Robert", , "alexis.robert@lshtm.ac.uk", role = "ctb"),
    person("Niel", "Hens", role = "ctb"),
    person("Pietro", "Coletti", role = c("col", "dtm")),
    person("Lloyd", "Chapman", , "l.chapman4@lancaster.ac.uk", role = "ctb")
  )
Description: Methods for sampling contact matrices from diary data for
    use in infectious disease modelling, as discussed in Mossong et al.
    (2008) <doi:10.1371/journal.pmed.0050074>.
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Imports: checkmate, countrycode, curl, data.table, grDevices, httr,
        jsonlite, lifecycle, lubridate, memoise, purrr, oai, wpp2017,
        xml2, cli, rlang, methods
Suggests: contactsurveys, ggplot2, here, knitr, quarto, reshape2,
        rmarkdown, roxyglobals (>= 1.0.0), testthat, withr
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
RoxygenNote: 7.3.3
URL: https://github.com/epiforecasts/socialmixr,
        https://epiforecasts.io/socialmixr/
BugReports: https://github.com/epiforecasts/socialmixr/issues
Config/testthat/edition: 3
Packaged: 2026-04-28 17:43:28 UTC; sebfunk
Author: Sebastian Funk [aut, cre],
  Lander Willem [aut],
  Hugo Gruson [aut],
  Nicholas Tierney [aut] (ORCID: <https://orcid.org/0000-0003-1460-8722>),
  Maria Bekker-Nielsen Dunbar [ctb],
  Carl A. B. Pearson [ctb],
  Sam Clifford [ctb],
  Christopher Jarvis [ctb],
  Alexis Robert [ctb],
  Niel Hens [ctb],
  Pietro Coletti [col, dtm],
  Lloyd Chapman [ctb]
Maintainer: Sebastian Funk <sebastian.funk@lshtm.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-29 06:40:18 UTC
Built: R 4.5.2; ; 2026-05-01 18:05:45 UTC; unix
