# Generated by roxygen2: do not edit by hand

S3method("[",contact_survey)
S3method(as.matrix,contact_matrix)
S3method(check,contact_survey)
S3method(clean,contact_survey)
S3method(plot,contact_matrix)
S3method(print,contact_matrix)
export(agegroups_to_limits)
export(as_contact_survey)
export(assign_age_groups)
export(check)
export(clean)
export(compute_matrix)
export(contact_age_distribution)
export(contact_matrix)
export(download_survey)
export(get_citation)
export(get_survey)
export(limits_to_agegroups)
export(list_surveys)
export(load_survey)
export(matrix_plot)
export(per_capita)
export(pop_age)
export(reduce_agegroups)
export(split_matrix)
export(survey)
export(survey_countries)
export(survey_country_population)
export(symmetrise)
export(weigh)
export(wpp_age)
export(wpp_countries)
import(data.table)
import(wpp2017)
importFrom(checkmate,assert_character)
importFrom(checkmate,assert_data_frame)
importFrom(checkmate,assert_list)
importFrom(checkmate,assert_names)
importFrom(countrycode,countrycode)
importFrom(curl,curl_download)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(data.table,fcase)
importFrom(data.table,fread)
importFrom(data.table,melt)
importFrom(data.table,rbindlist)
importFrom(data.table,setkey)
importFrom(data.table,setkeyv)
importFrom(data.table,uniqueN)
importFrom(grDevices,heat.colors)
importFrom(graphics,axis)
importFrom(graphics,image)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(httr,GET)
importFrom(httr,config)
importFrom(httr,content)
importFrom(httr,http_error)
importFrom(httr,status_code)
importFrom(httr,user_agent)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecated)
importFrom(lubridate,period)
importFrom(lubridate,period_to_seconds)
importFrom(lubridate,years)
importFrom(memoise,memoise)
importFrom(methods,formalArgs)
importFrom(oai,list_records)
importFrom(purrr,walk)
importFrom(rlang,"%||%")
importFrom(stats,median)
importFrom(stats,runif)
importFrom(stats,xtabs)
importFrom(utils,bibentry)
importFrom(utils,data)
importFrom(utils,globalVariables)
importFrom(utils,hasName)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_text)
