.socialmixr.env <- new.env(parent = emptyenv())
assign("cached_functions", list(), envir = .socialmixr.env)
## usethis namespace: start
#' @importFrom lifecycle deprecated
#' @importFrom lifecycle deprecate_soft
## usethis namespace: end
NULL
