# survey_countries() sends deprecation warning

    `survey_countries()` was deprecated in socialmixr 0.5.0.
    i Please use `contactsurveys::download_survey()` instead.
    i We recommend using contactsurveys::download_survey() to download your surveys, and then you can load them with socialmixr::load_survey() and explore which countries are in the data.

