% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-utils.R
\name{warn_multiple_observations}
\alias{warn_multiple_observations}
\title{Warn if survey has multiple observations per participant}
\usage{
warn_multiple_observations(
  participants,
  observation_key = NULL,
  filter_hint = c("pipeline", "legacy")
)
}
\arguments{
\item{participants}{participant data.table}

\item{observation_key}{optional column name(s) identifying observations}

\item{filter_hint}{character; "pipeline" for pipeline-style hint or
"legacy" for contact_matrix-style hint}
}
\value{
NULL invisibly
}
\description{
Issues a warning when a survey contains multiple observations per
participant (more rows than unique part_id values).
}
\keyword{internal}
