% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset-contact-survey.R
\name{assemble_survey}
\alias{assemble_survey}
\title{Assemble a contact survey with new participant/contact data}
\usage{
assemble_survey(x, participants, contacts)
}
\arguments{
\item{x}{a \code{contact_survey} object}

\item{participants}{new participants data.table}

\item{contacts}{new contacts data.table}
}
\value{
a \code{contact_survey} object with all fields from \code{x} preserved
}
\description{
Creates a new survey object preserving all fields from the original,
replacing only \code{participants} and \code{contacts} with the supplied data.
}
\keyword{internal}
