% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{.get_survey}
\alias{.get_survey}
\title{Internal function to get survey data}
\usage{
.get_survey(survey, ...)
}
\arguments{
\item{survey}{a DOI or url to get the survey from, or a \code{\link[=survey]{survey()}} object.}

\item{...}{currently unused}
}
\description{
Internal function to get survey data
}
\keyword{internal}
