% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survey.R
\name{survey}
\alias{survey}
\title{Contact survey}
\usage{
survey(participants, contacts, reference = NULL)
}
\arguments{
\item{participants}{a \code{data.frame} containing information on participants}

\item{contacts}{a \code{data.frame} containing information on contacts}

\item{reference}{a \code{list} containing information needed to
reference the survey, in particular it can contain a "title",
"bibtype", "author", "doi", "publisher", "note", "year"}
}
\value{
a new survey object
}
\description{
Deprecated. Use \code{as_survey} instead.
}
\author{
Sebastian Funk
}
