% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lists.R
\name{wpp_countries}
\alias{wpp_countries}
\title{List all countries and regions for which socialmixr has population data}
\usage{
wpp_countries()
}
\value{
list of countries
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

This function is deprecated in favour of passing population data directly
to \code{\link[=contact_matrix]{contact_matrix()}} via the \code{survey_pop} argument, which removes the need
for a country list. Additionally, the underlying \code{wpp2017} data is outdated.
For countries available in more recent WPP editions, use the \code{wpp2024}
package from GitHub.
}
\details{
Uses the World Population Prospects data from the \code{wpp2017} package.
}
\examples{
if (requireNamespace("wpp2017", quietly = TRUE)) {
  wpp_countries()
}
}
