% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset-contact-survey.R
\name{[.contact_survey}
\alias{[.contact_survey}
\title{Subset a contact survey}
\usage{
\method{[}{contact_survey}(x, i, ...)
}
\arguments{
\item{x}{a \code{contact_survey} object}

\item{i}{an expression to evaluate as a row filter (e.g.
\code{country == "United Kingdom"})}

\item{...}{ignored}
}
\value{
a filtered \code{contact_survey} object
}
\description{
Filters a \code{contact_survey} object using an expression. The expression is
evaluated against whichever table(s) contain the referenced columns
(participants, contacts, or both). When participants are filtered, contacts
are automatically pruned to matching \code{part_id}s.
}
\examples{
data(polymod)
polymod[country == "United Kingdom"]

}
