% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-result.R
\name{new_contact_matrix}
\alias{new_contact_matrix}
\title{Create a contact_matrix object}
\usage{
new_contact_matrix(matrix, participants, ...)
}
\arguments{
\item{matrix}{a numeric matrix with age group dimnames}

\item{participants}{a data.frame with columns \code{age.group}, \code{participants},
\code{proportion}}

\item{...}{additional named elements (e.g. \code{mean.contacts}, \code{normalisation},
\code{contacts} from \code{\link[=split_matrix]{split_matrix()}})}
}
\value{
a \code{contact_matrix} object (an S3 class inheriting from \code{list})
}
\description{
Create a contact_matrix object
}
\keyword{internal}
